{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvAddPrinter.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvAddPrinter;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, 
  JvBaseDlg, JvTypes, Shlobj, Activex, ShellApi;

type
  TJvAddPrinter = class(TJvCommonDialogP)
  private
  protected
  public
  published
    procedure Execute;override;
  end;

implementation

{**************************************************}
function ItemIDListCreate(const Allocator:IMalloc; const size:integer):pItemIDList;
var
   NewItemIDList:  pItemIDList;
begin
   NewItemIDList := Allocator.Alloc(size);
   if NewItemIDList<>nil then
      ZeroMemory(NewItemIDList, size);
   result := NewItemIDList
end;
{**************************************************}
function ItemIDListGetNextItem(const ItemIDList:pItemIDList):pItemIDList;
begin
   if ItemIDlist=nil then
     result:=nil
   else
     result:= Pointer(integer(ItemIDList) + ItemIDList.mkid.cb);
end;
{**************************************************}
function ItemIDListGetSize(const ItemIDList:pItemIDList):Integer;
var
   TempItemIDList:  pItemIDList;
begin
   result:=0;
   TempItemIDList := ItemIDList;
   if TempItemIDList <> nil then
   begin
      while (TempItemIDList.mkid.cb > 0) do
      begin
         inc(result, TempItemIDList.mkid.cb);
         TempItemIDList := ItemIDListGetNextItem(TempItemIDList)
      end;
      inc(result, 2*SizeOf(byte))
   end;
end;
{**************************************************}
function ItemIDListsConcatenate(const Allocator:  IMalloc;const List1, List2:  pItemIDList):  pItemIDlist;
var
   List1Length  :  Integer;
   List2Length  :  Integer;
   NewItemIDList:  pItemIDList;
begin
   List1Length := 0;
   if List1<>nil then
      List1Length := ItemIDListGetSize(List1) - 2*SizeOf(BYTE);
   List2Length := ItemIDListGetSize(List2);
   NewItemIDList := ItemIDListCreate(Allocator, List1Length + List2Length);
   if NewItemIDList<>nil then
   begin
      if List1<>nil then
         CopyMemory(NewItemIDList, List1, List1Length);
      CopyMemory(Pointer(integer(NewItemIDList) + List1Length),List2, List2Length)
   end;
   result := NewItemIDList
end;
{**************************************************}
function GetPrinterItemIDList(const DeskTopFolder:  IShellFolder):  pItemIDlist;
var
   hOK              :  hResult;
   PrinterItemIDList:  pItemIDList;
begin
   if DeskTopFolder=nil then
        RESULT:=nil
   else
   begin
      hOK:=SHGetSpecialFolderLocation(0,CSIDL_PRINTERS,PrinterItemIDList);
      if Failed(hOk) then
         result:=nil
      else
         result:=PrinterItemIDList;
   end;
end;
{**************************************************}
function GetAddPrinterItem(const Allocator:  IMalloc):  pItemIDList;
var
   DeskTopFolder    :  IShellFolder;
   EnumIDList       :  IEnumIDList;
   hOK              :  hResult;
   PrinterItemIDList:  pItemIDList;
   PrintersFolder   :  IShellFolder;
   Retrieved        :  Integer;
   TargetItemIDList :  pItemIDList;
   TempItemIDList   :  pItemIDList;
begin
   TargetItemIDList:=nil;
   if Allocator<>nil then
   begin
      if Succeeded(SHGetDesktopFolder(DeskTopFolder)) then
      begin
         PrinterItemIDList := GetPrinterItemIDList(DeskTopFolder);
         if PrinterItemIDList<>nil then
         begin
            hOK := DeskTopFolder.BindToObject(PrinterItemIDList,NIL,IID_IShellFolder,Pointer(PrintersFolder));
            if Succeeded(hOK) then
            begin
               if Succeeded(PrintersFolder.EnumObjects(0, SHCONTF_FOLDERS OR SHCONTF_NONFOLDERS,EnumIDList)) then
               begin
                  hOK:=EnumIDList.Next(1, TempItemIDList,Cardinal(Retrieved));
                  if (Retrieved>0) and Succeeded(hOK) then
                     TargetItemIDList := ItemIDListsConcatenate(Allocator,PrinterItemIDList,TempItemIDList);
               end;
            end;
         end;
      end;
   end;
   result:=TargetItemIDList
end;
{**************************************************}
procedure TJvAddPrinter.Execute;
Var
   AddPrinterItemIDList:  pItemIDList;
   Allocator           :  IMalloc;
   ShellExecuteInfo    :  TShellExecuteInfo;
begin
   if CoGetMalloc(MEMCTX_TASK,Allocator)=S_OK then
   begin
      AddPrinterItemIDList := GetAddPrinterItem(Allocator);
      try
        if AddPrinterItemIDList<>nil then
        begin
           ZeroMemory(@ShellExecuteInfo, SizeOf(TShellExecuteInfo));
           with ShellExecuteInfo do
           begin
              cbSize   := SizeOf(TShellExecuteInfo);
              fMask    := SEE_MASK_INVOKEIDLIST OR SEE_MASK_FLAG_NO_UI;
              lpIDList := AddPrinterItemIDlist;
              nShow    := SW_SHOWDEFAULT
           end;
           ShellExecuteEx(@ShellExecuteInfo);
        end;
      finally
        Allocator.Free(AddPrinterItemIDList)
      end;
   end;
end;
{**************************************************}
end.
