{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvAni.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvAni;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Consts, JvFunctions;

type
  TAniHeader = record
     dwSizeof: LongInt;
     dwFrames: LongInt;
     dwSteps: LongInt;
     dwCX: LongInt;
     dwCY: LongInt;
     dwBitCount: LongInt;
     dwPlanes: LongInt;
     dwJIFRate: LongInt;
     dwFlags: LongInt;
  end;
  {$EXTERNALSYM TAniHeader}

  TJvAni = class (TGraphic)
  private
    FTransparent: Boolean;
    FAuthor: string;
    FTitle: string;
    FNumberFrames: Cardinal;
    FHeader: TAniHeader;
    FCurrentIcon: TIcon;
    FImage:TMemoryStream;
    FRate, FSequence, FImages:TList;
    FIndex: Integer;
    FTimer: TTimer;

    procedure Clear;
    procedure SetIndex(const Value: integer);
    function GetAnimated: boolean;
    procedure SetAnimated(const Value: boolean);
    procedure CalcDelay;
  protected
    function GetEmpty: Boolean; override;
    function GetHeight: Integer; override;
    function GetWidth: Integer; override;
    procedure SetHeight (Value: integer); override;
    procedure SetWidth (Value: integer); override;
    procedure Animate(Sender: TObject);
    procedure SetTransparent(Value: Boolean); override;
    function GetTransparent: Boolean; override;
  public
    constructor Create; override;
    destructor Destroy; override;
    procedure Assign (Source: TPersistent); override;
    procedure LoadFromStream (Stream: TStream); override;
    procedure SaveToStream (Stream: TStream); override;
    procedure LoadFromClipboardFormat (AFormat: Word; AData: THandle; APalette: HPalette); override;
    procedure SaveToClipboardFormat (var Format: Word; var Data: THandle; var APalette: HPalette); override;
    procedure Draw (ACanvas: TCanvas; const Rect: TRect); override;

    property Author: string read FAuthor;
    property Title: string read FTitle;
    property Icon:TIcon read FCurrentIcon;
    property FramesCount:Cardinal read FNumberFrames default 0;
    property Index:Integer read FIndex write SetIndex default -1;
    property Animated:boolean read GetAnimated write SetAnimated default false;
  end;

implementation

resourcestring
  RC_AniExtension        =     'ani';
  RC_AniFilterName       =     'ANI Image';

{**************************************************}
constructor TJvAni.Create;
begin
  inherited;
  FTransparent  := False;
  FAuthor       := '';
  FTitle        := '';
  FNumberFrames := 0;
  FImage        := TMemoryStream.Create;
  FRate         := TList.Create;
  FSequence     := TList.Create;
  FImages       := TList.Create;
  FCurrentIcon  := TIcon.Create;
  FIndex        := -1;
  FTimer        := TTimer.Create(nil);
  FTimer.Interval:= 100;
  FTimer.OnTimer:=Animate;
  FTimer.Enabled:= false;
end;
{**************************************************}
destructor TJvAni.Destroy;
begin
  FTimer.Free;
  Clear;
  FImage.Free;
  FRate.Free;
  FSequence.Free;
  FImages.Free;
  FCurrentIcon.Free;

  inherited;
end;
{**************************************************}
procedure TJvAni.Clear;
begin
  FImages.Clear;
  FAuthor       := '';
  FTitle        := '';
  FNumberFrames := 0;
  FRate.Clear;
  FSequence.Clear;
  FImage.Size   := 0;
  if FCurrentIcon.Handle<>0 then DestroyIcon(FCurrentIcon.Handle);
  FCurrentIcon.Handle:=0;
  FIndex:=-1;

  if not(csDestroying in Application.ComponentState) then
    Changed(self);
end;
{**************************************************}
procedure TJvAni.Assign (Source: TPersistent);
var
 stream:TStream;
begin
  if Source = Nil then
    Clear
  else
    if Source is TJvAni then
    begin
      stream:=TMemoryStream.Create;
      TJvAni(Source).SaveToStream(stream);
      stream.Position:=0;
      LoadFromStream(stream);
      stream.Free;
      Animated:=TJvAni(Source).Animated;
    end
    else
      inherited Assign(Source);
end;
{**************************************************}
function TJvAni.GetEmpty:boolean;
begin
  Result := FNumberFrames = 0;
end;
{**************************************************}
procedure TJvAni.SetHeight(Value: integer);
begin
  raise EInvalidGraphicOperation.Create(sChangeIconSize);
end;
{**************************************************}
procedure TJvAni.SetWidth(Value: integer);
begin
  raise EInvalidGraphicOperation.Create(sChangeIconSize);
end;
{**************************************************}
function TJvAni.GetWidth: Integer;
begin
  Result := FHeader.dwCX;
end;
{**************************************************}
function TJvAni.GetHeight: Integer;
begin
  Result := FHeader.dwCY;
end;
{**************************************************}
procedure TJvAni.LoadFromClipboardFormat (AFormat: Word; AData: THandle; APalette: HPalette);
begin
   raise EInvalidGraphicOperation.Create(sIconToClipboard);
end;
{**************************************************}
procedure TJvAni.SaveToClipboardFormat (var Format: Word; var Data: THandle; var APalette: HPalette);
begin
   raise EInvalidGraphicOperation.Create(sIconToClipboard);
end;
{**************************************************}
procedure TJvAni.LoadFromStream (Stream: TStream);
const
  an_RIFF = $46464952;
  an_ACON = $4E4F4341;
  an_LIST = $5453494C;
  an_INFO = $4F464E49;
  an_INAM = $4D414E49;
  an_IART = $54524149;
  an_anih = $68696E61;
  an_rate = $65746172;
  an_fram = $00617266;
  an_icon = $6E6F6369;
  an_seq  = $20716573;
var
  dw,dw2:Integer;

  procedure Error;
  begin
    raise EInvalidGraphic.Create ('Animated icon image is not valid');
  end;

  function ReadByte:byte;
  begin
    dec(dw,1);
    if FImage.Read(result,sizeof(result))=0 then
      result:=0;
  end;

  function ReadDWord:DWord;
  var
   d:DWord;
  begin
    dec(dw,4);
    if FImage.Read(d,sizeof(d))<4 then d:=0;
    result:=d;
  end;

  function ReadString:string;
  var
   p:Pchar;
   l:Integer;
  begin
    l:=ReadDWord;

    //Check for integrity of the data
    if (l>0)and(l<FImage.Size-FImage.Position) then
    begin
      dec(dw,l);
      GetMem(p,l+1);
      ZeroMemory(p,l+1);
      FImage.Read(p^,l);
      result:=StrPas(p);
      FreeMem(p);
      if ReadByte<>$00 then
        FImage.Position:=FImage.Position-1;
    end
    else
      result:='';
  end;

  procedure ReadList(List: TList);
  var
   Len: Integer;
  begin
    Len:= ReadDWord div 4;
    if (Len=FHeader.dwSteps) then
    begin
      for Len:=1 to Len do
        List.Add(Pointer(ReadDWord));
    end
    else
      Error;
  end;

  procedure ReadFrames;
  var
   i,j,k:Integer;
  begin
    for i:=0 to FHeader.dwFrames-1 do
      if ReadDWord<>an_icon then Error
      else
      begin
        FImages.Add(Pointer(FImage.Position));
        k:=ReadDWord;
        dec(dw,k);
        j:=FImage.Position+k;
        if (FHeader.dwCX=0)or(FHeader.dwCY=0) then
        begin
          FImage.Position:=FImage.Position+6;
          FHeader.dwCX:=ReadByte;
          FHeader.dwCY:=ReadByte;
        end;
        FImage.Position:=j;
      end;
  end;

begin
  Clear;
  FImage.CopyFrom(Stream,Stream.Size-Stream.Position);
  FImage.Position:=0;
  if ReadDWord<>an_RIFF then Error;
  FImage.Size:=ReadDWord;
  if ReadDWord<>an_ACON then Error;

  while FImage.Position<FImage.Size do
    case ReadDWord of
      an_LIST :  begin
                   dw:=ReadDWord;
                   while dw>0 do
                   begin
                     dw2:=ReadDWord;
                     case dw2 of
                        an_INAM  : FTitle  := Readstring;
                        an_IART  : FAuthor := Readstring;
                        else
                           if (dw2 and $00FFFFFF)=an_fram then
                             ReadFrames;
                     end;
                   end;
                 end;
      an_anih :  FImage.Read(FHeader,ReadDWord);
      an_rate :  ReadList(FRate);
      an_seq  :  ReadList(FSequence);
      else
         FImage.Position:=FImage.Position+Integer(ReadDWord);
    end;

  FNumberFrames:=FHeader.dwFrames;
  FIndex:=-1;
  SetIndex(0);
end;
{**************************************************}
procedure TJvAni.SaveToStream (Stream: TStream);
begin
  if GetEmpty then
    raise EInvalidGraphicOperation.Create (sInvalidImage);
  Stream.Write(FImage.Memory^,FImage.Size)
end;
{**************************************************}
procedure TJvAni.Draw(ACanvas: TCanvas; const Rect: TRect);
var
  bmp:TBitmap;
begin
  if FCurrentIcon.Handle<>0 then
  begin
    bmp:=TBitmap.Create;
    with bmp do
    begin
      Transparent:=FTransparent;
      Width:=FHeader.dwCX;
      Height:=FHeader.dwCY;
      DrawIcon(Canvas.Handle, 0, 0, FCurrentIcon.Handle);
    end;
    ACanvas.Draw(Rect.Left,Rect.Top,bmp);
    bmp.Free;
  end;
end;
{**************************************************}
procedure TJvAni.SetIndex(const Value: integer);
type
  TIconHeader = packed record
     AlwaysZero: Word;
     CursorType: Word;
     NumIcons: Word;
  end;

  TIconDirEntry = packed record
     Width, Height, Colors: Byte;
     Reserved: Byte;
     dwReserved: LongInt;
     dwBytesInRes: LongInt;
     dwImageOffset: LongInt;
  end;
var
 p:Integer;
 Len:Integer;
 IconHeader: TIconHeader;
begin
  if (FImages.Count>0) and (Value>=0) and (Value<FHeader.dwFrames) then
   if FIndex<>Value then
   begin
     FIndex:=Value;
     if FCurrentIcon.Handle<>0 then
       DestroyIcon(FCurrentIcon.Handle);

     p:=integer(FImage.Memory);
     FImage.Position:=integer(FImages[FIndex]);
     FImage.Read(Len,Sizeof(Len));
     FImage.Read(IconHeader,SizeOf(IconHeader));
     FImage.Position:=FImage.Position+(Sizeof(TIconDirEntry)*IconHeader.NumIcons);
     dec(Len, sizeof (IconHeader) + (sizeof (TIconDirEntry) * IconHeader.NumIcons));
     inc(p,FImage.Position);

     p:=CreateIconFromResource(pointer(p),Len,True,$30000);
     FCurrentIcon.Handle:=p;
     Changed(Self);
   end;
end;
{**************************************************}
function TJvAni.GetAnimated: boolean;
begin
  result:=FTimer.Enabled;
end;
{**************************************************}
procedure TJvAni.SetAnimated(const Value: boolean);
begin
  if Value<>FTimer.Enabled then
    FTimer.Enabled:=Value;
end;
{**************************************************}
procedure TJvAni.Animate(Sender: TObject);
begin
  FTimer.Enabled:=false;
  if FNumberFrames>0 then
    SetIndex((FIndex+1)mod FNumberFrames);
  CalcDelay;
  FTimer.Enabled:=true;
end;
{**************************************************}
procedure TJvAni.CalcDelay;
begin
  if FIndex=-1 then
    SetAnimated(False)
  else
  begin
    if (FIndex>0) and (FRate.Count>FIndex) then
      FTimer.Interval:=cardinal(FRate[FIndex])*(1000 div 60)
    else
    begin
      FTimer.Interval:=FHeader.dwJIFRate*(1000 div 60);
      if FTimer.Interval=0 then
       FTimer.Interval:=100;
    end
  end;
end;
{**************************************************}
procedure TJvAni.SetTransparent(Value: Boolean);
begin
  if Value<>FTransparent then
  begin
    FTransparent := Value;
    Changed(Self);
  end;
end;
{**************************************************}
function TJvAni.GetTransparent: Boolean;
begin
  result:=FTransparent;
end;
{**************************************************}

initialization
  Registerclass(TJvAni);
  TPicture.RegisterFileFormat(RC_AniExtension, RC_AniFilterName, TJvAni);
finalization
  TPicture.UnRegisterGraphicclass(TJvAni);
end.



