{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvButtonPersistent.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvButtonPersistent;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, JvButton,
  JvSpeedButton, stdctrls, buttons;

type
  TJvButtonPersistent = class(TPersistent)
  private
    FVisible: boolean;
    FFlat: boolean;
    FEnabled: boolean;
    FCaption: string;
    FOnChanged: TNotifyEvent;
    FFontEffect: boolean;
    FHint: string;
    FShowHint: boolean;
    FFont: TFont;
    procedure SetCaption(const Value: string);
    procedure SetEnabled(const Value: boolean);
    procedure SetFlat(const Value: boolean);
    procedure SetVisible(const Value: boolean);
    procedure SetFontEffect(const Value: boolean);
    procedure SetHint(const Value: string);
    procedure SetShowHint(const Value: boolean);
    procedure SetFont(const Value: TFont);
  protected
    procedure Changed;
  public
    procedure AssignTo(Sender: TObject);
    property OnChanged:TNotifyEvent read FOnChanged write FOnChanged;
    constructor Create;
  published
    property Enabled:boolean read FEnabled write SetEnabled default true;
    property Visible:boolean read FVisible write SetVisible default true;
    property Caption:string read FCaption write SetCaption;
    property Flat:boolean read FFlat write SetFlat default true;
    property HotTrack:boolean read FFontEffect write SetFontEffect default true;
    property HotTrackFont:TFont read FFont write SetFont;
    property Hint:string read FHint write SetHint;
    property ShowHint:boolean read FShowHint write SetShowHint;
  end;

implementation

{***********************************************}
procedure TJvButtonPersistent.AssignTo(Sender: TObject);
begin
  if Sender is TButton then
  begin
    TButton(Sender).Enabled := Enabled;
    TButton(Sender).Visible := Visible;
    TButton(Sender).Caption := Caption;
    TButton(Sender).Hint := Hint;
    TButton(Sender).ShowHint := ShowHint;
    if Sender is TJvButton then
    begin
      TJvButton(Sender).HotTrack := HotTrack;
      TJvButton(Sender).HotTrackFont.Assign(FFont);
    end;
  end
  else if Sender is TSpeedButton then
  begin
    TSpeedButton(Sender).Enabled := Enabled;
    TSpeedButton(Sender).Visible := Visible;
    TSpeedButton(Sender).Caption := Caption;
    TSpeedButton(Sender).Hint := Hint;
    TSpeedButton(Sender).ShowHint := ShowHint;
    TSpeedButton(Sender).Flat := Flat;
    if Sender is TJvSpeedButton then
    begin
      TJvSpeedButton(Sender).HotTrack := HotTrack;
      TJvSpeedButton(Sender).HotTrackFont.Assign(FFont);
    end;
  end;
end;
{***********************************************}
procedure TJvButtonPersistent.Changed;
begin
  if Assigned(FOnChanged) then
    FOnChanged(self);
end;
{***********************************************}
constructor TJvButtonPersistent.Create;
begin
  FFont := TFont.Create;
  FEnabled:=true;
  FFlat:=true;
  FVisible:=true;
  FFontEffect:=true;
end;
{***********************************************}
procedure TJvButtonPersistent.SetCaption(const Value: string);
begin
  FCaption := Value;
  Changed;
end;
{***********************************************}
procedure TJvButtonPersistent.SetEnabled(const Value: boolean);
begin
  FEnabled := Value;
  Changed;
end;
{***********************************************}
procedure TJvButtonPersistent.SetFlat(const Value: boolean);
begin
  FFlat := Value;
  Changed;
end;
{***********************************************}
procedure TJvButtonPersistent.SetFont(const Value: TFont);
begin
  FFont.Assign(Value);
  Changed;
end;
{***********************************************}
procedure TJvButtonPersistent.SetFontEffect(const Value: boolean);
begin
  FFontEffect := Value;
  Changed;
end;
{***********************************************}
procedure TJvButtonPersistent.SetHint(const Value: string);
begin
  FHint := Value;
  Changed;
end;
{***********************************************}
procedure TJvButtonPersistent.SetShowHint(const Value: boolean);
begin
  FShowHint := Value;
  Changed;
end;
{***********************************************}
procedure TJvButtonPersistent.SetVisible(const Value: boolean);
begin
  FVisible := Value;
  Changed;
end;
{***********************************************}
end.
