{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvCombobox.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvCombobox;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, StdCtrls,
  JvMaxPixel, JvPropAutoSave, JvItemsSearchs;

type
  TJvCombobox = class(TComboBox)
  private
    FKey:Word;
    FAutoComplete: boolean;
    FSearching: boolean;
    FOnMouseEnter: TNotifyEvent;
    FColor,FSaved: TColor;
    FOnMouseLeave: TNotifyEvent;
    FOnCtl3DChanged: TNotifyEvent;
    FOnParentColorChanged: TNotifyEvent;
    FOnRestored: TNotifyEvent;
    FOver: Boolean;
    FMaxPixel: TJvMaxPixel;
    FAutoSave: TJvAutoSave;
    FItemSearchs: TJvItemsSearchs; 
    procedure MaxPixelChanged(Sender: TObject);
  protected
    procedure Change;override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorCHANGED;
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
    procedure Loaded;override;
    function SearchExactString(Value: string; CaseSensitive: Boolean = true):Integer;
    function SearchPrefix(Value: string; CaseSensitive: Boolean = true):Integer;
    function SearchSubString(Value: string; CaseSensitive: Boolean = true):Integer;
    function DeleteExactString(Value: string; All: Boolean;
      CaseSensitive: Boolean = true):Integer;
  published
    property SelStart;
    property SelText;
    property SelLength;
    property ItemIndex;

    property AutoComplete : boolean read FAutoComplete write FAutoComplete default true;
    property AutoSave: TJvAutoSave read FAutoSave write FAutoSave;
    property HintColor:TColor read FColor write FColor default clInfoBk;
    property MaxPixel: TJvMaxPixel read FMaxPixel write FMaxPixel;

    property OnRestored:TNotifyEvent read FOnRestored write FOnRestored;
    property OnMouseEnter: TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave: TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property OnCtl3DChanged: TNotifyEvent read FonCtl3DChanged write FonCtl3DChanged;
    property OnParentColorChange: TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
  end;

implementation

{**************************************************}
constructor TJvCombobox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FColor := clInfoBk;
  FAutoComplete := true;
  FSearching := false;
  Controlstyle := Controlstyle + [csAcceptsControls];
  FOver := false;
  FMaxPixel := TJvMaxPixel.Create(self);
  FMaxPixel.OnChanged := MaxPixelChanged;
  FAutoSave := TJvAutoSave.Create(self);
  FItemSearchs := TJvItemsSearchs.Create;
end;
{**************************************************}
destructor TJvCombobox.Destroy;
begin
  FMaxPixel.Free;
  FAutoSave.Free;
  FItemSearchs.Free;
  inherited;
end;
{**************************************************}
procedure TJvCombobox.Loaded;
var
 st: string;
 i: Integer;
begin
  inherited;
  if Style = csDropDownList then
  begin
    if FAutoSave.LoadValue(i) then
    begin
      ItemIndex := i;
      if Assigned(FOnRestored) then
        FOnRestored(self);
    end;
  end
  else
  begin
    if (FAutoSave.LoadValue(st))  then
    begin
      Text := st;
      if Assigned(FOnRestored) then
        FOnRestored(self);
    end;
  end;
end;
{**************************************************}
procedure TJvCombobox.Change;
var
 res: Integer;
 st: string;
 start,finish: Integer;
begin
  inherited;
  if not(FSearching) and (FAutoComplete) then
  begin
    st := Text;
    FMaxPixel.Test(st,Font);
    if Text<>st then
    begin
      Text := st;
      Exit;
    end;
    if (FKey<>8) and (FKey<>46) and (FKey<>13) then
    begin
      FSearching := true;
      st := text;
      res := SendMessage(handle,CB_FINDstring,-1,longint(pchar(st)));
      if res<>CB_ERR then
        try
          itemindex := res;
          start := length(st);
          finish := length(items[res])-start;
          text := items[res];
          SelStart := start;
          SelLength := finish;
        except
        end;
      FSearching:=false;
    end;
  end;
  if Style = csDropDownList then
    FAutoSave.SaveValue(ItemIndex)
  else
    FAutoSave.SaveValue(Text);
end;
{**************************************************}
function TJvCombobox.SearchExactString(Value: string;
  CaseSensitive: Boolean): Integer;
begin
  result := FItemSearchs.SearchExactString(Items,Value,CaseSensitive);
end;
{**************************************************}
function TJvCombobox.SearchPrefix(Value: string;
  CaseSensitive: Boolean):Integer;
begin
  result := FItemSearchs.SearchPrefix(Items,Value,CaseSensitive);
end;
{**************************************************}
procedure TJvCombobox.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited Keydown(Key,shift);
  FKey := Key;
end;
{**************************************************}
procedure TJvCombobox.CMCtl3DChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonCtl3DChanged) then
    FOnCtl3DChanged(self);
end;
{**************************************************}
procedure TJvCombobox.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonParentColorChanged) then
    FOnParentColorChanged(Self);
end;
{**************************************************}
procedure TJvCombobox.CMMouseEnter(var Message: TMessage);
begin
  if not FOver then
  begin
    FSaved := Application.HintColor;
    Application.HintColor := FColor;
    FOver := true;
  end;
  if Assigned(FonMouseEnter) then
    FOnMouseEnter(self);
end;
{**************************************************}
procedure TJvCombobox.CMMouseLeave(var Message: TMessage);
begin
  if FOver then
  begin
    Application.HintColor := FSaved;
    FOver := false;
  end;
  if Assigned(FonMouseLeave) then
    FOnMouseLeave(self);
end;
{**************************************************}
function TJvCombobox.SearchSubString(Value: string;
  CaseSensitive: Boolean): Integer;
begin
  result := FItemSearchs.SearchSubString(Items,Value,CaseSensitive);
end;
{**************************************************}
function TJvCombobox.DeleteExactString(Value: string; All: Boolean;
  CaseSensitive: Boolean): Integer;
begin
  result := FItemSearchs.DeleteExactString(Items,Value,CaseSensitive);
end;
{**************************************************}
procedure TJvCombobox.MaxPixelChanged(Sender: TObject);
var
 st: string;
begin
  if Style<>csDropDownList then
  begin
    st := Text;
    FMaxPixel.Test(st,Font);
    if Text<>st then
      Text := st;
    SelStart := Length(Text);
  end;
end;
{**************************************************}

end.
