{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvDeviceChanged.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvDeviceChanged;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, JvTypes, Forms;

type
  TJvDeviceChanged = class(TComponent)
  private
    FHandle:THandle;
    FOnDeviceArrived:TOnDeviceArrived;
    FOnDeviceRemoveCompleted:TOnDeviceArrived;
    function GetFirstDriveLetter(unitmask:longint):char;
  protected
  public
    procedure WndProc(var Msg: TMessage);
    procedure WMDeviceChange(var Msg : TWMDeviceChange);
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property OnDeviceArrived:TOnDeviceArrived read FOnDeviceArrived write FOnDeviceArrived;
    property OnDeviceRemoveCompleted:TOnDeviceArrived read FOnDeviceRemoveCompleted write FOnDeviceRemoveCompleted;
  end;

implementation

{************************************************************}
constructor TJvDeviceChanged.Create(AOwner: TComponent);
begin
   inherited;
   FHandle:=AllocateHWND(WndProc);
end;
{************************************************************}
destructor TJvDeviceChanged.Destroy;
begin
   DeallocateHWnd(FHandle);
   inherited;
end;
{************************************************************}
procedure TJvDeviceChanged.WndProc(var Msg: TMessage);
begin
   if (Msg.Msg = WM_DEVICECHANGE) then
      WMDeviceChange(TWMDeviceChange(Msg))
   else
      Msg.Result := DefWindowProc(FHandle, Msg.Msg, Msg.wParam, Msg.lParam);
end;
{************************************************************}
function TJvDeviceChanged.GetFirstDriveLetter(unitmask:longint):char;
var
   DriveLetter:shortint;
begin
   DriveLetter := Ord('A');
   while (unitmask and 1)=0  do
   begin
      unitmask := unitmask shr 1;
      inc(DriveLetter);
   end;
   Result:=Char(DriveLetter);
end;
{************************************************************}
procedure TJvDeviceChanged.WMDeviceChange(var Msg : TWMDeviceChange);
var
   lpdb : PDEV_BROADCAST_HDR;
   lpdbv : PDEV_BROADCAST_VOLUME;
begin
   lpdb := PDEV_BROADCAST_HDR(Msg.dwData);
   case Msg.Event of
       DBT_DEVICEARRIVAL :
       begin
          if lpdb^.dbch_devicetype = DBT_DEVTYP_VOLUME then
          begin
             lpdbv := PDEV_BROADCAST_VOLUME(Msg.dwData);
             if (lpdbv^.dbcv_flags and DBTF_MEDIA) = 1 then
                if Assigned(FOnDeviceArrived) then
                   FOnDeviceArrived(Self, GetFirstDriveLetter(lpdbv^.dbcv_unitmask));
          end;
       end;
       DBT_DEVICEREMOVECOMPLETE :
       begin
          if lpdb^.dbch_devicetype = DBT_DEVTYP_VOLUME then
          begin
             lpdbv := PDEV_BROADCAST_VOLUME(Msg.dwData);
             if (lpdbv^.dbcv_flags and DBTF_MEDIA) = 1 then
                if Assigned(FOnDeviceRemoveCompleted) then
                    FOnDeviceRemoveCompleted(Self, GetFirstDriveLetter(lpdbv^.dbcv_unitmask));
          end;
       end;
   end
end;
{************************************************************}
end.
