{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvFavoritesButton.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvFavoritesButton;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, JvTypes, JvButton, Menus, FileCtrl, JvDirectories,
  Inifiles, JvFunctions, ShellApi, imglist;

type
  TJvFavoritesButton = class(TJvButton)
  private
    FPopup: TPopupMenu;
    FDirs: TJvDirectories;
    FOnUrl: TOnUrlClick;
    FOnPopup: TNotifyEvent;
    FImages: TImageList;
    procedure UrlClick(Sender: TObject);
  protected
    procedure DeleteItem(Item: TMenuItem; LookTag: boolean=false);
    procedure PopupCreate(Sender: TObject);
    procedure DirectoryClick(Sender: TObject);
    procedure DynBuild(Item: TMenuItem; Directory: string);
    procedure AddIconFrom(Path: string);
  public
    procedure Click; override;
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property OnUrlClick:TOnUrlClick read FOnUrl write FOnUrl;
    property OnPopup:TNotifyEvent read FOnPopup write FOnPopup;
  end;

implementation

resourcestring
  RC_EmptyItem       =    '<Empty>';

{*******************************************************}
constructor TJvFavoritesButton.Create(AOwner: TComponent);
var
 it: TMenuItem;
begin
  inherited;
  FDirs := TJvDirectories.Create(self);

  //Create Popup
  FPopup := TPopupMenu.Create(self);
  it := TMenuItem.Create(FPopup);
  with it do
  begin
    Enabled := false;
    Caption := RC_EmptyItem;
    Tag := 1;
  end;
  FPopup.Items.Add(it);
  FPopup.OnPopup := PopupCreate;

  //Create Images
  FImages := TImageList.Create(self);
  FImages.Width := 16;
  FImages.Height := 16;
  FImages.DrawingStyle := dsTransparent;
  FPopup.Images := FImages;
  AddIconFrom(FDirs.WindowsDirectory);
end;
{*******************************************************}
destructor TJvFavoritesButton.Destroy;
begin
  FDirs.free;
  DeleteItem(FPopup.Items);
  FPopup.free;
  inherited;
end;
{*******************************************************}
procedure TJvFavoritesButton.Click;
var
 p: TPoint;
begin
  inherited;
  p.x := 0;
  p.y := Height;
  p := ClientToScreen(p);
  FPopup.Popup(p.x,p.y);
  if Assigned(FOnPopup) then
    FOnPopup(self);
end;
{*******************************************************}
procedure TJvFavoritesButton.UrlClick(Sender: TObject);
begin
  if Assigned(FOnUrl) then
    FOnUrl(self,(Sender as TMenuItem).Hint);
end;
{*******************************************************}
procedure TJvFavoritesButton.DeleteItem(Item: TMenuItem; LookTag: boolean);
var
 i: Integer;
begin
  for i:=Item.Count-1 downto 0 do
    if (not LookTag) or (Item[i].Tag = 0) then
    begin
      DeleteItem(Item[i]);
      Item[i].Free;
    end;
end;
{*******************************************************}
procedure TJvFavoritesButton.PopupCreate(Sender: TObject);
begin
  DynBuild(FPopup.Items,FDirs.Favorites);
end;
{*******************************************************}
procedure TJvFavoritesButton.DynBuild(Item: TMenuItem; Directory: string);
var
 res: Integer;
 SearchRec: TSearchRec;
 it,it2: TMenuItem;
 first: boolean;
begin
  DeleteItem(Item, true);
  if (Directory<>'') and (Directory[Length(Directory)]<>'\') then
    Directory := Directory+'\';
  res := FindFirst(Directory+'*.*', faAnyFile, SearchRec);
  first := true;
  while res=0 do
  begin
    if (SearchRec.Name<>'.') and (SearchRec.Name<>'..') then
    begin
      if (SearchRec.Attr and faDirectory = faDirectory) then
      begin
        if first then
          Item.Items[0].Visible := false;
        it := TMenuItem.Create(Item);
        it.Caption := SearchRec.Name;
        it.Hint := Directory+SearchRec.Name;
        it.OnClick := DirectoryClick;
        it.ImageIndex := 0;
        Item.Add(it);
        it2 := TMenuItem.Create(it);
        with it2 do
        begin
          Caption := RC_EmptyItem;
          Enabled := false;
          Tag := 1;
        end;
        it.Add(it2);
      end
      else if UpperCase(ExtractFileExt(SearchRec.Name))='.URL' then
      begin
        if first then
          Item.Items[0].Visible := false;
        if FImages.Count=1 then
          AddIconFrom(Directory+SearchRec.Name);
        it := TMenuItem.Create(Item);
        it.Caption := ChangeFileExt(SearchRec.Name,'');;
        it.OnClick := UrlClick;
        it.Hint := Directory+SearchRec.Name;
        it.ImageIndex := 1;
        Item.Add(it);
      end;
    end;
    res := FindNext(SearchRec);
  end;
  FindClose(SearchRec);
end;
{*******************************************************}
procedure TJvFavoritesButton.DirectoryClick(Sender: TObject);
begin
  DynBuild((Sender as TMenuItem),(Sender as TMenuItem).Hint);
end;
{*******************************************************}
procedure TJvFavoritesButton.AddIconFrom(Path: string);
var
 FileInfo : SHFILEINFO;
 bmp,bmp2: TBitmap;
begin
  bmp := TBitmap.Create;
  bmp.Width := 32;
  bmp.Height := 32;
  SHGetFileInfo(PChar(Path),0,FileInfo,SizeOf(FileInfo),SHGFI_ICON);
  DrawIcon(bmp.Canvas.Handle,0,0,FileInfo.hIcon);
  bmp2 := TBitmap.Create;
  bmp2.Width := 16;
  bmp2.Height := 16;
  bmp2.Canvas.StretchDraw(Rect(0,0,16,16),bmp);
  FImages.Add(bmp2,nil);
  bmp2.Free;
  bmp.Free;
end;
{*******************************************************}
end.
