{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvFileUtils.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvFileUtils;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  JvTypes, ShellApi;

type
  TJvFilesUtils = class(TComponent)
  private
    FTitle:string;
    FAllowUndo:boolean;
    FOnCollision:boolean;
    FConfirmation:boolean;
    FSilent : boolean;
    FFilesOnly:boolean;
  protected
  public
  published
    property Title: string read Ftitle write FTitle;
    property AllowUndo : boolean read FAllowUndo write FAllowUndo;
    property RenameOnCollision : boolean read FOnCollision write FOnCollision;
    property Confirmation : boolean read FConfirmation write FConfirmation;
    property Silent:boolean read FSilent write FSilent;
    property FilesOnly : boolean read FFilesOnly write FFilesOnly;
    function DeleteFile(FileName:string):boolean;
    procedure DeleteFileToBIn(FileName:string);
    function CopyFile(From,Destination:string):boolean;
    function MoveFile(From,Destination:string):boolean;
    function RenameFile(From,Destination:string):boolean;
  end;

implementation

{**********************************************************}
function TJvFilesUtils.DeleteFile(FileName:string):boolean;
Var
   SH:TSHFileOpStruct;
begin
   with SH do
   begin
      Wnd:=Application.handle;
      wFunc:=FO_DELETE;
      pFrom:=Pchar(FileName);
      lpszProgressTitle:=pchar(FTitle);
      fFlags:=0;
      if FAllowundo then fFlags:=fFlags+FOF_ALLOWUNDO;
      if FOnCollision then fFlags:=fFlags+FOF_RENAMEONCOLLISION;
      if not(FConfirmation) then fFlags:=fFlags+FOF_NOCONFIRMATION;
      if FSilent then fFlags:=fFlags+FOF_SILENT;
      if FFilesOnly then fFlags:=fFlags+FOF_FILESONLY;
   end;
   result:=SHFileOperation(SH)=0;
end;
{**********************************************************}
procedure TJvFilesUtils.DeleteFileToBIn(FileName:string);
Var
   SH:TSHFileOpStruct;
begin
   with SH do
   begin
      Wnd:=0;
      wFunc:=FO_DELETE;
      pFrom:=Pchar(FileName);
      fFlags:=FOF_ALLOWUNDO
   end;
   SHFileOperation(SH);
end;
{**********************************************************}
function TJvFilesUtils.CopyFile(From,Destination:string):boolean;
var
   SH:TSHFileOpStruct;
begin
   with SH do
   begin
      Wnd:=Application.handle;
      wFunc:=FO_Copy;
      pFrom:=Pchar(From);
      pTo:=Pchar(Destination);
      lpszProgressTitle:=pchar(FTitle);
      fFlags:=0;
      if FAllowundo then fFlags:=fFlags+FOF_ALLOWUNDO;
      if FOnCollision then fFlags:=fFlags+FOF_RENAMEONCOLLISION;
      if not(FConfirmation) then fFlags:=fFlags+FOF_NOCONFIRMATION;
      if FSilent then fFlags:=fFlags+FOF_SILENT;
      if FFilesOnly then fFlags:=fFlags+FOF_FILESONLY;
   end;
   result:=SHFileOperation(SH)=0;
end;
{**********************************************************}
function TJvFilesUtils.MoveFile(From,Destination:string):boolean;
var
   SH:TSHFileOpStruct;
begin
   with SH do
   begin
      Wnd:=Application.handle;
      wFunc:=FO_MOVE;
      pFrom:=Pchar(From);
      pTo:=Pchar(Destination);
      lpszProgressTitle:=pchar(FTitle);
      fFlags:=0;
      if FAllowundo then fFlags:=fFlags+FOF_ALLOWUNDO;
      if FOnCollision then fFlags:=fFlags+FOF_RENAMEONCOLLISION;
      if not(FConfirmation) then fFlags:=fFlags+FOF_NOCONFIRMATION;
      if FSilent then fFlags:=fFlags+FOF_SILENT;
      if FFilesOnly then fFlags:=fFlags+FOF_FILESONLY;
   end;
   result:=SHFileOperation(SH)=0;
end;
{**********************************************************}
function TJvFilesUtils.RenameFile(From,Destination:string):boolean;
var
   SH:TSHFileOpStruct;
begin
   with SH do
   begin
      Wnd:=Application.handle;
      wFunc:=FO_RENAME;
      pFrom:=Pchar(From);
      pTo:=Pchar(Destination);
      lpszProgressTitle:=pchar(FTitle);
      fFlags:=0;
      if FAllowundo then fFlags:=fFlags+FOF_ALLOWUNDO;
      if FOnCollision then fFlags:=fFlags+FOF_RENAMEONCOLLISION;
      if not(FConfirmation) then fFlags:=fFlags+FOF_NOCONFIRMATION;
      if FSilent then fFlags:=fFlags+FOF_SILENT;
      if FFilesOnly then fFlags:=fFlags+FOF_FILESONLY;
   end;
   result:=SHFileOperation(SH)=0;
end;
{**********************************************************}
end.
