{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvGenetic.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvGenetic;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, JvTypes;

type
  TOnTest = function (Sender: TObject;Index:Integer;Member:pointer):byte of object;

  TJvGenetic = class(TComponent)
  private
    FMembers:TstringList;
    FGeneration: Integer;
    FSize: Integer;
    FNumbers: Integer;
    FOnTest: TOnTest;
    FCrossover: real;
    FMutate: real;
    procedure SetNumbers(const Value: integer);
    procedure SetSize(const Value: integer);
    procedure KillThemAll(Value:TstringList);
    function Generate(Pere,Mere:pointer;Size:integer):pointer;
    function Mutate(Value:byte):byte;
    function DoCrossover:boolean;
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;

    procedure NewGeneration;
    procedure NextGeneration;
    function GetMember(Index:integer):pointer;
    function GetAverage:real;
  published
    property Generation:Integer read FGeneration;
    property MemberSize:Integer read FSize write SetSize default 4;
    property Numbers:Integer read FNumbers write SetNumbers default 10;
    property OnTestMember:TOnTest read FOnTest write FOnTest;
    property CrossoverProbability:real read FCrossover write FCrossover;
    property MutateProbability:real read FMutate write FMutate;
  end;

implementation

type
  TGeneticMember = class
     Points:Cardinal;
     Data:pointer;
  end;

resourcestring
  RC_NoTest          =    'TJvGenetic : OnTestMember must be Assigned !';

{***************************************************}
constructor TJvGenetic.Create(AOwner: TComponent);
begin
  inherited;

  FMembers := TStringList.Create;
  Randomize;
  FGeneration := 0;
  FNumbers := 10;
  FSize := 4;
  FCrossover := 0.6;
  FMutate := 0.003;
end;
{***************************************************}
destructor TJvGenetic.Destroy;
begin
  KillThemAll(FMembers);
  FMembers.free;
  inherited;
end;
{***************************************************}
function TJvGenetic.DoCrossover: boolean;
begin
  result := Random<FCrossover;
end;
{***************************************************}
function TJvGenetic.Generate(Pere, Mere: pointer; Size: integer): pointer;
var
  i,count:Integer;
  p,s:pointer;
begin
   if DoCrossover then
      count:=random(Size-1)
   else
      count:=Size;

   result:=AllocMem(Size);
   p:=result;
   s:=Pere;
   for i:=0 to count-1 do
   begin
      byte(p^):=Mutate(byte(s^));
      p:=pointer(integer(p)+1);
      s:=pointer(integer(s)+1);
   end;
   s:=Mere;
   s:=Pointer(integer(s)+count);
   for i:=count to Size-1 do
   begin
      byte(p^):=Mutate(byte(s^));
      p:=pointer(integer(p)+1);
      s:=pointer(integer(s)+1);
   end;
end;
{***************************************************}
function TJvGenetic.GetAverage: real;
var
  i:Integer;
begin
   result:=0;
   if FMembers.count<>0 then
   begin
      for i:=0 to FMembers.count-1 do
         result:=result+TGeneticMember(FMembers.Objects[i]).Points;
      result:=result / FMembers.count;
   end;
end;
{***************************************************}
function TJvGenetic.GetMember(Index: integer): pointer;
begin
   result:=TGeneticMember(FMembers.Objects[Index]).Data;
end;
{***************************************************}
procedure TJvGenetic.KillThemAll(Value:TstringList);
begin
   while Value.Count>0 do
   begin
      FreeMem(TGeneticMember(Value.Objects[0]).Data,FSize);
      TGeneticMember(Value.Objects[0]).Free;
      Value.Delete(0);
   end;
end;
{***************************************************}
function TJvGenetic.Mutate(Value:byte):byte;
var
  b:byte;
  i:Integer;
begin
   b:=$80;
   result:=Value;
   for i:=0 to 7 do
   begin
      if (Random<FMutate) then
      begin
         if (result and b)=0 then result:=result or b
         else result:=result and (not b);
      end;
      b:=b shr 1;
   end;
end;
{***************************************************}
procedure TJvGenetic.NewGeneration;
var
  i,j:Integer;
  Member:TGeneticMember;
  p:pointer;
begin
   if (FNumbers>0)and(FSize>0) then
   begin
      KillThemAll(FMembers);
      FGeneration:=0;
      for i:=0 to FNumbers-1 do
      begin
         Member:=TGeneticMember.Create;
         Member.Data:=AllocMem(FSize);
         p:=Member.Data;
         for j:=0 to FSize-1 do
         begin
            byte(p^):=Random(256);
            p:=pointer(integer(p)+1);
         end;
         if not(Assigned(FOnTest)) then
            raise Exception.Create(RC_NoTest);
         Member.Points:=FOnTest(self,i,Member.Data);
         FMembers.AddObject('',TObject(Member));
      end;
   end;
end;
{***************************************************}
procedure TJvGenetic.NextGeneration;
var
  a,b,tot:cardinal;
  i:Integer;
  pere,mere:Integer;
  FGenerat:TstringList;

  Member:TGeneticMember;
begin
   if (FNumbers>0)and(FSize>0) then
   begin
      inc(FGeneration);

      //Compute the sum of Points
      tot:=0;
      for i:=0 to FNumbers-1 do
          inc(tot,TGeneticMember(FMembers.Objects[i]).Points);

      //Create new Generation
      FGenerat:=TstringList.Create;
      for i:=0 to FNumbers do
      begin
         a:=random(tot);
         b:=TGeneticMember(FMembers.Objects[0]).Points;
         pere:=0;
         while b<a do
         begin
            inc(pere);
            inc(b,TGeneticMember(FMembers.Objects[pere]).Points);
         end;

         a:=random(tot);
         b:=TGeneticMember(FMembers.Objects[0]).Points;
         mere:=0;
         while b<a do
         begin
            inc(mere);
            inc(b,TGeneticMember(FMembers.Objects[mere]).Points);
         end;

         //Copy, Crossover and mutate
         Member:=TGeneticMember.Create;
         Member.Data:=Generate(TGeneticMember(FMembers.Objects[mere]).Data,TGeneticMember(FMembers.Objects[pere]).Data,FSize);

         if Assigned(FOnTest) then
            Member.Points:=FOnTest(self,i,Member.Data)
         else
            raise Exception.Create(RC_NoTest);

         //Add new element to FGenerat
         FGenerat.AddObject('',TObject(Member));
      end;
      KillThemAll(FMembers);
      FMembers.Assign(FGenerat);
      FGenerat.Free;
   end;
end;
{***************************************************}
procedure TJvGenetic.SetNumbers(const Value: integer);
begin
   if FNumbers<>Value then
   begin
      FNumbers := Value;
      KillThemAll(FMembers);
      FGeneration:=0;
   end;
end;
{***************************************************}
procedure TJvGenetic.SetSize(const Value: integer);
begin
   if FSize<>Value then
   begin
      FSize := Value;
      KillThemAll(FMembers);
      FGeneration:=0;
   end;
end;
{***************************************************}

end.
