{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvGradient.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvGradient;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  JvTypes;

type
  TJvGradient = class(TGraphicControl)
  private
     Fstyle:TGradstyle;
     FStarTColor:TColor;
     FEndColor:TColor;
     FSteps:word;
     FOldB:TBitmap;
     FOldX,FOldY:Integer;
     procedure SetSteps(Value:word);
     procedure SetStarTColor(Value:TColor);
     procedure SetEndColor(Value:TColor);
     procedure Setstyle(Value:TGradstyle);
  protected
    procedure Paint;override;
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property Align;
    property ShowHint;
    property Visible;
    property ParentShowHint;
    property Enabled;
    property PopupMenu;
    property style:TGradstyle read Fstyle write Setstyle default grHorizontal;
    property StarTColor:TColor read FStarTColor write SetStarTColor default clBlue;
    property EndColor:TColor read FEndColor write SetEndColor default clBlack;
    property Steps:word read FSteps write SetSteps default 100;
  end;

implementation

{***********************************************}
constructor TJvGradient.Create(AOwner: TComponent);
begin
   FOldX:=0;
   FOldY:=0;
   FSteps:=100;
   FOldB:=TBitmap.Create;
   Fstyle:=grHorizontal;
   FEndColor:=clBlack;
   FStarTColor:=clblue;
   inherited;
   Align:=AlClient;
end;
{***********************************************}
destructor TJvGradient.Destroy;
begin
   FOldB.free;
   inherited;
end;
{***********************************************}
procedure TJvGradient.Paint;
var
   bt:TBitmap;
   i:Integer;
   j,k:real;
   Deltas:array [0..2] of real;  //R,G,B
   r:TRect;
   FStart,FEnd:TColor;
begin
   if csDestroying in ComponentState then exit;
   if (FOldX<>self.Width)or(FOldY<>self.Height) then
   begin
      if (FStarTColor<0) then FStart:=GetSysColor(FStarTColor and not( $80000000))
      else FStart:=FStarTColor;
      if (FEndColor<0) then FEnd:=GetSysColor(FEndColor and not( $80000000))
      else FEnd:=FEndColor;

      FOldX:=self.width;
      FOldY:=self.Height;
      bt:=TBitmap.Create;
      bt.Width:=self.width;
      bt.Height:=self.height;
      case Fstyle of
         grFilled:
                  begin
                     bt.Canvas.brush.Color:=FStart;
                     bt.Canvas.brush.style:=bsSolid;
                     bt.Canvas.FillRect(Rect(0,0,self.width,self.height));
                  end;
         grEllipse:
                  begin
                     bt.Canvas.brush.Color:=FStart;
                     bt.Canvas.brush.style:=bsSolid;
                     bt.Canvas.FillRect(Rect(0,0,self.width,self.height));
                     if FSteps>(self.width div 2) then FSteps:=trunc(self.Width/2);
                     if FSteps>(self.height div 2) then FSteps:=trunc(self.height/2);
                     Deltas[0]:=(GetRValue(FEnd)-GetRValue(FStart))/FSteps;
                     Deltas[1]:=(GetGValue(FEnd)-GetGValue(FStart))/FSteps;
                     Deltas[2]:=(GetBValue(FEnd)-GetBValue(FStart))/FSteps;
                     bt.Canvas.brush.style:=bsSolid;
                     j:=(Self.Width/FSteps)/2;
                     k:=(Self.height/FSteps)/2;
                     for i:=0 to FSteps do
                     begin
                        r.top:=round(i*k);
                        r.bottom:=self.height-r.top;
                        r.right:=round(i*j);
                        r.left:=self.width-r.right;
                        bt.Canvas.brush.Color:=RGB(Round(GetRValue(FStart)+i*Deltas[0]),Round(GetGValue(FStart)+i*Deltas[1]),Round(GetBValue(FStart)+i*Deltas[2]));
                        bt.Canvas.pen.Color:=bt.canvas.brush.color;
                        bt.Canvas.Ellipse(r.right,r.top,r.left,r.bottom);
                     end;
                  end;
         grHorizontal:
                  begin
                     if FSteps>self.width then FSteps:=self.Width;
                     Deltas[0]:=(GetRValue(FEnd)-GetRValue(FStart))/FSteps;
                     Deltas[1]:=(GetGValue(FEnd)-GetGValue(FStart))/FSteps;
                     Deltas[2]:=(GetBValue(FEnd)-GetBValue(FStart))/FSteps;
                     bt.Canvas.brush.style:=bsSolid;
                     j:=Self.Width/FSteps;
                     for i:=0 to FSteps do
                     begin
                        r.Top:=0;
                        r.Bottom:=self.height;
                        r.Left:=round(i*j);
                        r.Right:=round((i+1)*j);
                        bt.Canvas.brush.Color:=RGB(Round(GetRValue(FStart)+i*Deltas[0]),Round(GetGValue(FStart)+i*Deltas[1]),Round(GetBValue(FStart)+i*Deltas[2]));
                        bt.Canvas.FillRect(r);
                     end;
                  end;
         grVertical:
                  begin
                     if FSteps>self.height then FSteps:=self.Height;
                     Deltas[0]:=(GetRValue(FEnd)-GetRValue(FStart))/FSteps;
                     Deltas[1]:=(GetGValue(FEnd)-GetGValue(FStart))/FSteps;
                     Deltas[2]:=(GetBValue(FEnd)-GetBValue(FStart))/FSteps;
                     bt.Canvas.brush.style:=bsSolid;
                     j:=Self.Height/FSteps;
                     for i:=0 to FSteps do
                     begin
                        r.left:=self.width;
                        r.right:=0;
                        r.top:=round(i*j);
                        r.Bottom:=round((i+1)*j);
                        bt.Canvas.brush.Color:=RGB(Round(GetRValue(FStart)+i*Deltas[0]),Round(GetGValue(FStart)+i*Deltas[1]),Round(GetBValue(FStart)+i*Deltas[2]));
                        bt.Canvas.FillRect(r);
                     end;
                  end;
         grMount:
                  begin
                     bt.Canvas.brush.Color:=FStart;
                     bt.Canvas.brush.style:=bsSolid;
                     bt.Canvas.FillRect(Rect(0,0,self.width,self.height));
                     if FSteps>(self.width div 2) then FSteps:=trunc(self.Width/2);
                     if FSteps>(self.height div 2) then FSteps:=trunc(self.height/2);
                     Deltas[0]:=(GetRValue(FEnd)-GetRValue(FStart))/FSteps;
                     Deltas[1]:=(GetGValue(FEnd)-GetGValue(FStart))/FSteps;
                     Deltas[2]:=(GetBValue(FEnd)-GetBValue(FStart))/FSteps;
                     bt.Canvas.brush.style:=bsSolid;
                     j:=(Self.Width/FSteps)/2;
                     k:=(Self.height/FSteps)/2;
                     for i:=0 to FSteps do
                     begin
                        r.top:=round(i*k);
                        r.bottom:=self.height-r.top;
                        r.right:=round(i*j);
                        r.left:=self.width-r.right;
                        bt.Canvas.brush.Color:=RGB(Round(GetRValue(FStart)+i*Deltas[0]),Round(GetGValue(FStart)+i*Deltas[1]),Round(GetBValue(FStart)+i*Deltas[2]));
                        bt.Canvas.pen.Color:=bt.canvas.brush.color;
                        bt.Canvas.RoundRect(r.right,r.top,r.left,r.bottom,((r.left-r.right)div 2),((r.bottom-r.top)div 2));
                     end;
                  end;
         grPyramid:
                  begin
                     bt.Canvas.brush.Color:=FStart;
                     bt.Canvas.brush.style:=bsSolid;
                     bt.Canvas.FillRect(Rect(0,0,self.width,self.height));
                     if FSteps>(self.width div 2) then FSteps:=trunc(self.Width/2);
                     if FSteps>(self.height div 2) then FSteps:=trunc(self.height/2);
                     Deltas[0]:=(GetRValue(FEnd)-GetRValue(FStart))/FSteps;
                     Deltas[1]:=(GetGValue(FEnd)-GetGValue(FStart))/FSteps;
                     Deltas[2]:=(GetBValue(FEnd)-GetBValue(FStart))/FSteps;
                     bt.Canvas.brush.style:=bsSolid;
                     j:=(Self.Width/FSteps)/2;
                     k:=(Self.height/FSteps)/2;
                     for i:=0 to FSteps do
                     begin
                        r.top:=round(i*k);
                        r.bottom:=self.height-r.top;
                        r.right:=round(i*j);
                        r.left:=self.width-r.right;
                        bt.Canvas.brush.Color:=RGB(Round(GetRValue(FStart)+i*Deltas[0]),Round(GetGValue(FStart)+i*Deltas[1]),Round(GetBValue(FStart)+i*Deltas[2]));
                        bt.Canvas.pen.Color:=bt.canvas.brush.color;
                        bt.Canvas.FillRect(Rect(r.right,r.top,r.left,r.bottom));
                     end;
                  end;
      end;
      FOldB.Assign(bt);
      bt.free;
   end;
   self.Canvas.Draw(0,0,FOldB);
end;
{***********************************************}
procedure TJvGradient.Setstyle(Value:TGradstyle);
begin
   Fstyle:=Value;
   FOldX:=0;
   self.paint;
end;
{***********************************************}
procedure TJvGradient.SetStarTColor(Value:TColor);
begin
   FStarTColor:=Value;
   FOldX:=0;
   self.paint;
end;
{***********************************************}
procedure TJvGradient.SetSteps(Value:word);
begin
   FSteps:=Value;
   FOldX:=0;
   self.paint;
end;
{***********************************************}
procedure TJvGradient.SetEndColor(Value:TColor);
begin
   FEndColor:=Value;
   FOldX:=0;
   self.paint;
end;
{***********************************************}
end.
