{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvHtmlParser.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvHtmlParser;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, JvTypes;

type
  TJvHtmlParser = class(TComponent)
  private
    FOnKeyFound:TOnKeyFound;
    FInfos:TParserInfos;
    FKeys:TstringList;
    FFile:TFileName;
    procedure SetInfos(Value:TParserInfos);
    procedure SetFile(Value:TFileName);
    function FirstLine(var Value:string):string;
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property OnKeyFound:TOnKeyFound read FOnKeyFound write FOnKeyFound;
    property FileName:TFileName read Ffile write SetFile;
    property Parser:TParserInfos read FInfos write SetInfos;
    procedure AnalyseFile;
    procedure AddCondition(Keyword,StartTag,EndTag:string);
    procedure RemoveCondition(index:integer);
    procedure ClearConditions;
    procedure GetCondition(Index:Integer;var Keyword:string;var StartTag:string;var EndTag:string);
  end;

implementation

{*************************************************}
constructor TJvHtmlParser.Create(AOwner: TComponent);
begin
   Finfos:=TstringList.Create;
   FKeys:=TstringList.Create;
   inherited;
   SetInfos(Finfos);
end;
{*************************************************}
destructor TJvHtmlParser.Destroy;
begin
   FInfos.free;
   FKeys.free;
   inherited;
end;
{*************************************************}
procedure TJvHtmlParser.SetFile(Value:TFileName);
begin
   Ffile:=Value;
   AnalyseFile;
end;
{*************************************************}
function TJvHtmlParser.FirstLine(var Value:string):string;
begin
   if pos(#10,Value)=0 then
   begin
      result:=Value;
      Value:='';
   end
   else
   begin
      result:=Copy(Value,1,pos(#10,Value)-1);
      Value:=Copy(Value,pos(#10,Value)+1,length(Value));
      if pos(#13,Value)=1 then Value:=Copy(Value,2,length(Value));
   end;
end;
{*************************************************}
procedure TJvHtmlParser.SetInfos(Value:TParserInfos);
var
   i:Integer;
   ob:TParserInf;
   cap:string;
begin
   FInfos:=Value;
   FKeys.clear;
   i:=0;
   while (i<Finfos.count) do
   begin
      ob:=TParserInf.Create;
      try
         cap:=Finfos[i];
         inc(i);
         ob.StartTag:=Finfos[i];
         inc(i);
         ob.EndTag:=Finfos[i];
         inc(i);
         ob.MustBe:=StrToInt(Finfos[i]);
         inc(i);
         ob.Take:=StrToInt(Finfos[i]);
         inc(i);
      finally
         Fkeys.addobject(cap,TObject(ob));
      end;
   end;
end;
{*************************************************}
procedure TJvHtmlParser.AnalyseFile;
var
   fich:textfile;
   buf,st,st2:string;
   i,j:Integer;
begin
   if (Fkeys.count=0)and(FInfos.count<>0) then SetInfos(Finfos);
   if Fkeys.count>0 then
   begin
      try
         assignfile(fich,FileName);
         reset(fich);
         buf:='';
         while (not(eof(fich))or(buf<>'')) do
         begin
            if buf='' then
            begin
               readln(fich,buf);
               st:=FirstLine(buf);
            end
            else st:=FirstLine(buf);

            for i:=0 to Fkeys.count-1 do
            begin
               j:=pos(TParserInf(FKeys.objects[i]).StartTag,st);
               if (j=TParserInf(FKeys.objects[i]).MustBe) or ((j<>0)and(TParserInf(FKeys.objects[i]).MustBe=-1)) then
               begin
                  case TParserInf(FKeys.objects[i]).Take of
                      0 : begin
                             st2:=Copy(st,j+length(TParserInf(FKeys.objects[i]).StartTag),length(st));
                             j:=pos(TParserInf(FKeys.objects[i]).EndTag,st2);
                             if j<>0 then
                                  st2:=Copy(st2,1,j-1);
                          end;
                      1 : st2:=Copy(st,1,j-1);
                      2 : st2:=Copy(st,j+length(TParserInf(FKeys.objects[i]).StartTag),length(st));
                      3 : st2:=st;
                  end;
                  //Between limits
                  //All before start tag
                  //All after start tag
                  //The whole line if containing start tag
                  if Assigned(FOnKeyFound) then
                     FOnKeyFound(self,Fkeys[i],st2,st);
               end;
            end;
         end;
      finally
         CloseFile(Fich);
      end;
   end;
end;
{*************************************************}
procedure TJvHtmlParser.AddCondition(Keyword,StartTag,EndTag:string);
var
   ob:TParserInf;
begin
   ob:=TParserInf.Create;
   ob.StartTag:=StartTag;
   ob.EndTag:=EndTag;
   Fkeys.addobject(Keyword,TObject(ob));
end;
{*************************************************}
procedure TJvHtmlParser.RemoveCondition(index:integer);
begin
   Fkeys.Delete(index);
end;
{*************************************************}
procedure TJvHtmlParser.ClearConditions;
begin
   FInfos.clear;
   FKeys.clear;
end;
{*************************************************}
procedure TJvHtmlParser.GetCondition(Index:Integer;var Keyword:string;var StartTag:string;var EndTag:string);
begin
   Keyword:=Fkeys[index];
   StartTag:=TParserInf(Fkeys.objects[Index]).StartTag;
   EndTag:=TParserInf(Fkeys.objects[Index]).EndTag;
end;
{*************************************************}
end.
