{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvPlaylist.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvPlaylist;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, JvListBox2;

type
  TJvPlaylist = class(TJvListbox2)
  private
    FShow: boolean;
    FItems: TStringList;
    FShowExt: boolean;
    FRefresh: boolean;
    procedure SetShow(const Value: boolean);
    procedure SetItems(const Value: TStringList);
    procedure SetShowExt(const Value: boolean);
  protected
    procedure LBDeleteString(var Message: TMessage); message LB_DELETEstring;
    procedure Changed;override;

    function GetPath(Value: string; Position: integer): string;
    procedure Refresh;
    procedure ItemsChanged(Sender: TObject);
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;

    procedure AddItem(FileName: string);
    procedure AddItems(Value: TStrings);
    function GetItem(Index: integer):string;
    procedure DeleteDeadFiles;
    procedure SortBySongName;
    procedure SortByPath;
    procedure SortByPathInverted;
    procedure SortBySongNameInverted;
    procedure ReverseOrder;
    procedure RandomOrder;
    procedure MoveSelectedUp;override;
    procedure MoveSelectedDown;override;
    procedure SavePlaylist(FileName: string);
    procedure LoadPlaylist(FileName: string);
  published
    property ShowNumbers:boolean read FShow write SetShow default false;
    property ShowExtension:boolean read FShowExt write SetShowExt default false; 
    property Items:TStringList read FItems write SetItems;
  end;

implementation

{**************************************************}
constructor TJvPlaylist.Create(AOwner: TComponent);
begin
  inherited;
  FShow:=false;
  FShowExt := false;
  FRefresh := false;
  FItems := TStringList.Create;
  FItems.OnChange := ItemsChanged;
end;
{**************************************************}
destructor TJvPlaylist.Destroy;
begin
  FItems.Free;
  inherited;
end;
{**************************************************}
function TJvPlayList.GetPath(Value: string; Position: integer):string;
begin
  result := ExtractFileName(Value);
  if not FShowExt then
    result := ChangeFileExt(result,'');
  if FShow then
    result := IntToStr(Position+1)+'. '+result;
end;
{**************************************************}
procedure TJvPlaylist.AddItem(FileName: string);
begin
  Items.Add(FileName);
end;
{**************************************************}
procedure TJvPlaylist.AddItems(Value: TStrings);
begin
  Items.Text := Items.Text + Value.Text;
end;
{**************************************************}
function TJvPlaylist.GetItem(index: Integer):string;
begin
  result := Items[index];
end;
{**************************************************}
procedure TJvPlaylist.DeleteDeadFiles;
var
 i: Integer;
begin
  for i:=Items.Count-1 downto 0 do
    if not FileExists(Items[i]) then
      Items.Delete(i);
end;
{**************************************************}
procedure TJvPlaylist.SortBySongName;
var
 a,b,c:Integer;
begin
  FRefresh := true;
  for a:=0 to Items.Count-1 do
  begin
    c := a;
    for b:=a to Items.Count-1 do
      if ExtractFileName(Items[b])<ExtractFileName(Items[c]) then
        c := b;
    Items.Exchange(a,c);
  end;
  FRefresh := false;
  Refresh;
end;
{**************************************************}
procedure TJvPlaylist.SortByPath;
begin
  FItems.Sort;
end;
{**************************************************}
procedure TJvPlaylist.SortByPathInverted;
begin
  FItems.Sort;
  ReverseOrder;
end;
{**************************************************}
procedure TJvPlaylist.SortBySongNameInverted;
begin
  SortBySongName;
  ReverseOrder;
end;
{**************************************************}
procedure TJvPlaylist.ReverseOrder;
var
 i,j: Integer;
begin
  j := FItems.Count-1;
  for i:=0 to ((FItems.Count div 2)-1) do
    FItems.Exchange(i,j-i);
end;
{**************************************************}
procedure TJvPlaylist.RandomOrder;
var
 i,j,k: Integer;
begin
  Randomize;
  for i:=0 to (FItems.Count div 2) do
  begin
    j := Random(FItems.Count);
    k := Random(FItems.Count);
    FItems.Exchange(j,k);
  end;
end;
{**************************************************}
procedure TJvPlaylist.SavePlaylist(FileName: string);
begin
  FItems.SaveToFile(FileName);
end;
{**************************************************}
procedure TJvPlaylist.LoadPlaylist(FileName: string);
var
 st,st2: string;
 i: Integer;
begin
  FItems.Clear;
  with TStringList.Create do
  begin
    LoadFromFile(FileName);
    for i:=0 to Count-1 do
    begin
      st := Strings[i];
      if Length(st)>0 then
        if st[1]<>'#' then
        begin
          st2 := ExtractFilePath(FileName);
          if st2[length(st2)]<>'\' then
            st2:=st2+'\';
          if ((not(FileExists(st))) or (pos('\',st)=0)) then
             if FileExists(st2+st) then
                st := st2+st;
          FItems.Add(st);
        end;
    end;
    Free;
  end;
end;
{**************************************************}
procedure TJvPlaylist.Refresh;
var
 i: Integer;
begin
  FRefresh := true;
  if Items.Count<>inherited Items.Count then
  begin
    inherited Items.Clear;
    for i:=0 to Items.Count-1 do
      inherited Items.Add(GetPath(Items[i],i));
  end
  else
    for i:=0 to Items.Count-1 do
      inherited Items[i] := GetPath(Items[i],i);
  FRefresh := false;
end;
{**************************************************}
procedure TJvPlaylist.SetShow(const Value: boolean);
begin
  if Value<>FShow then
  begin
    FShow := Value;
    Refresh;
  end;
end;
{**************************************************}
procedure TJvPlaylist.SetItems(const Value: TStringList);
begin
  FItems.Assign(Value);
  Refresh;
end;
{**************************************************}
procedure TJvPlaylist.SetShowExt(const Value: boolean);
begin
  if Value<>FShowExt then
  begin
    FShowExt := Value;
    Refresh;
  end;
end;
{**************************************************}
procedure TJvPlaylist.ItemsChanged(Sender: TObject);
begin
  Refresh;
end;
{**************************************************}
procedure TJvPlaylist.LBDeleteString(var Message: TMessage);
begin
  inherited;
  if not FRefresh then
  begin
    Items.OnChange := nil;
    Items.Delete(LongInt(Message.wParam));
    Items.OnChange := ItemsChanged;
  end;
end;
{**************************************************}
procedure TJvPlaylist.Changed;
begin
  Refresh;
end;
{**************************************************}
procedure TJvPlaylist.MoveSelectedDown;
var
 i: Integer;
begin
  if MultiSelect = false then
  begin
    if (ItemIndex<>-1) and (ItemIndex<Items.Count-1) then
    begin
      Items.Exchange(ItemIndex, ItemIndex+1);
      ItemIndex := ItemIndex+1;
    end;
    Exit;
  end;
  Items.OnChange := nil;
  FRefresh := true;
  if (Items.Count>0) and (SelCount>0) and (not(Selected[Items.Count-1])) then
  begin
    i := Items.Count-2;
    while i>=0 do
    begin
      if Selected[i] then
      begin
        Items.Exchange(i,i+1);
        Selected[i+1] := true;
      end;
      dec(i);
    end;
  end;
  FRefresh := false;
  Items.OnChange := ItemsChanged;
  Refresh;
end;
{**************************************************}
procedure TJvPlaylist.MoveSelectedUp;
var
 i:Integer;
begin
  if MultiSelect = false then
  begin
    if (ItemIndex>1) then
    begin
      Items.Exchange(ItemIndex, ItemIndex-1);
      ItemIndex := ItemIndex-1;
    end;
    Exit;
  end;
  Items.OnChange := nil;
  FRefresh := true;
  if (Items.Count>0) and (SelCount>0)and(not(Selected[0])) then
  begin
    i := 1;
    while i<Items.Count do
    begin
      if Selected[i] then
      begin
        Items.Exchange(i,i-1);
        Selected[i-1] := true;
      end;
      inc(i);
    end;
  end;
  FRefresh := false;
  Items.OnChange := ItemsChanged;
  Refresh;
end;
{**************************************************}
end.
