{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvRecentMenuBtn.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvRecentMenuBtn;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, JvButton, Menus, JvDirectories, JvTypes, FileCtrl, JvFunctions,
  ShellApi;

type
  TJvRecentMenuBtn = class(TJvButton)
  private
    FPopup:TPopupMenu;
    FDirs:TJvDirectories;
    FOnUrl:TOnLinkClick;
    FOnPopup: TNotifyEvent;
    procedure UrlClick(Sender: TObject);
  protected
    procedure CreatePopup(Sender: TObject);
    procedure DynBuild(Item: TMenuItem; Directory: string);
    procedure DeleteItem(Item: TMenuItem; LookTag: boolean=false);
  public
    procedure Click; override;
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property OnLinkClick:TOnLinkClick read FOnUrl write FOnUrl;
    property OnPopup:TNotifyEvent read FOnPopup write FOnPopup;
  end;

implementation

resourcestring
  RC_EmptyItem       =    '<Empty>';

{*******************************************************}
constructor TJvRecentMenuBtn.Create(AOwner: TComponent);
var
 it: TMenuItem;
begin
  inherited;
  FDirs := TJvDirectories.Create(self);

  //Create Popup
  FPopup := TPopupMenu.Create(self);
  it := TMenuItem.Create(FPopup);
  with it do
  begin
    Enabled := false;
    Caption := RC_EmptyItem;
    Tag := 1;
  end;
  FPopup.Items.Add(it);
  FPopup.OnPopup := CreatePopup;
end;
{*******************************************************}
destructor TJvRecentMenuBtn.Destroy;
begin
  FDirs.free;
  DeleteItem(FPopup.Items);
  FPopup.free;
  inherited;
end;
{*******************************************************}
procedure TJvRecentMenuBtn.Click;
var
 p: TPoint;
begin
  inherited;
  p.x := 0;
  p.y := Height;
  p := ClientToScreen(p);
  FPopup.Popup(p.x,p.y);
  if Assigned(FOnPopup) then
    FOnPopup(self);
end;
{*******************************************************}
procedure TJvRecentMenuBtn.UrlClick(Sender: TObject);
begin
  if Assigned(FOnUrl) then
    FOnUrl(self,(Sender as TMenuItem).hint);
end;
{*******************************************************}
procedure TJvRecentMenuBtn.CreatePopup(Sender: TObject);
begin
  DynBuild(FPopup.Items,FDirs.Recent);
end;
{*******************************************************}
procedure TJvRecentMenuBtn.DynBuild(Item: TMenuItem; Directory: string);
var
 res: Integer;
 SearchRec: TSearchRec;
 it: TMenuItem;
 w: word;
begin
  DeleteItem(Item, true);
  if (Directory<>'') and (Directory[Length(Directory)]<>'\') then
    Directory := Directory+'\';
  res := FindFirst(Directory+'*.*', faAnyFile, SearchRec);
  if Item.Count=1 then
    Item.Items[0].Visible := true;
  while (res=0) do
  begin
    if (SearchRec.Name<>'.') and (SearchRec.Name<>'..')  then
      if (SearchRec.Attr and faDirectory = 0) and (FileDateToDateTime(SearchRec.Time)>(Now-EncodeTime(23,0,0,0))) then
      begin
        if Item.Count=1 then
          Item.Items[0].Visible := false;
        it := TMenuItem.Create(Item);
        it.Caption := ChangeFileExt(SearchRec.Name,'');;
        it.OnClick := UrlClick;
        it.Hint := Directory+SearchRec.Name;
        w := 0;
        it.Bitmap.assign(IconToBitmap(ExtractAssociatedIcon(Application.handle,pchar(it.hint),w)));
        it.Bitmap.TransparentMode:=tmAuto;
        Item.Add(it);
      end;
    res := FindNext(SearchRec);
  end;
  FindClose(SearchRec);
end;
{*******************************************************}
procedure TJvRecentMenuBtn.DeleteItem(Item: TMenuItem; LookTag: boolean);
var
 i: Integer;
begin
  for i:=Item.Count-1 downto 0 do
    if (not LookTag) or (Item[i].Tag = 0) then
    begin
      DeleteItem(Item[i]);
      Item[i].Free;
    end;
end;
{*******************************************************}
end.
