{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvTrayIcon.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvTrayIcon;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, 
  JvTypes, ExtCtrls, Menus, ShellApi;

type
  TJvTrayIcon = class(TComponent)
  private
    FActive:boolean;
    FIcon:TIcon;
    Fic:TNotifyIconData;
    FHandle:THandle;
    FHint:string;
    FPopupMenu:TPopupMenu;
    FOnClick: TNotifyEvent;
    FOnDblClick: TNotifyEvent;
    FOnMouseMove : TNotifyEvent;
    FOnMouseUp : TNotifyEvent;
    FOnLBUTTONDOWN      :TNotifyEvent;
    FOnLBUTTONUP        :TNotifyEvent;
    FOnLBUTTONDBLCLK    :TNotifyEvent;
    FOnRBUTTONDOWN      :TNotifyEvent;
    FOnRBUTTONUP        :TNotifyEvent;
    FOnRBUTTONDBLCLK    :TNotifyEvent;
    FOnMBUTTONDOWN      :TNotifyEvent;
    FOnMBUTTONUP        :TNotifyEvent;
    FOnMBUTTONDBLCLK    :TNotifyEvent;
    FVisible:boolean;
    FAnimated: boolean;
    FDelay: Cardinal;
    FImgList: TimageList;
    FTimer:TTimer;
    FNumber:Integer;
    FDropDown: TPopupMenu;
    FTask: boolean;
    FRegisterServiceProcess:TRegisterServiceProcess;
    FDllHandle:THandle;
    procedure SetActive(Value:boolean);
    procedure Test(proc:TNotifyevent);
    procedure SetHint(Value:string);
    procedure SetIcon(Icon:TIcon);
    procedure SetVisible(Value:boolean);
    procedure OnAnimate(Sender: TObject);
    procedure SetAnimated(const Value: boolean);
    procedure SetDelay(const Value: Cardinal);
    procedure SetImgList(const Value: TimageList);
    procedure IconChanged(Sender: TObject);
    procedure SetTask(const Value: boolean);
  public
    procedure WndProc(var Message: TMessage);
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property Active:boolean read FActive write SetActive default false;
    property Icon:TIcon read FIcon write SetIcon;
    property Hint:string read FHint write SetHint;
    property PopupMenu:TPopupMenu read FPopupMenu write FPopupMenu;
    property ApplicationVisible:Boolean read FVisible write SetVisible default true;
    property Animated:boolean read FAnimated write SetAnimated default false;
    property Icons:TimageList read FImgList write SetImgList;
    property Delay:Cardinal read FDelay write SetDelay default 100;
    property OnClick: TNotifyEvent read FOnClick write FOnClick ;
    property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick ;
    property OnMouseMove : TNotifyEvent read FOnMouseMove write FOnMouseMove ;
    property OnMouseUp : TNotifyEvent read FOnMouseUp write FOnMouseUp ;
    property OnLeftButtonDown      :TNotifyEvent read FOnLBUTTONDOWN write FOnLBUTTONDOWN ;
    property OnLeftButtonUp        :TNotifyEvent read FOnLBUTTONUP write FOnLBUTTONUP ;
    property OnLeftButtonDblClick    :TNotifyEvent read FOnLBUTTONDBLCLK write FOnLBUTTONDBLCLK ;
    property OnRightButtonDown      :TNotifyEvent read FOnRBUTTONDOWN write FOnRBUTTONDOWN ;
    property OnRightButtonUp        :TNotifyEvent read FOnRBUTTONUP write FOnRBUTTONUP ;
    property OnRightButtonDblClick    :TNotifyEvent read FOnRBUTTONDBLCLK write FOnRBUTTONDBLCLK ;
    property OnMiddleButtonDown      :TNotifyEvent read FOnMBUTTONDOWN write FOnMBUTTONDOWN ;
    property OnMiddleButtonUp        :TNotifyEvent read FOnMBUTTONUP write FOnMBUTTONUP ;
    property OnMiddlebuttonDblClk    :TNotifyEvent read FOnMBUTTONDBLCLK write FOnMBUTTONDBLCLK ;
    procedure HideApplication;
    procedure ShowApplication;
    property DropDownMenu:TPopupMenu read FDropDown write FDropDown;
    property VisibleInTaskList:boolean read FTask write SetTask default true;
  end;

implementation

{**************************************************}
constructor TJvTrayIcon.Create(AOwner: TComponent);
begin
  inherited;
  FIcon:=TIcon.Create;
  if not(csDesigning in ComponentState) then
    FIcon.Assign(Application.Icon);
  FIcon.OnChange:=IconChanged;
  FVisible:=true;
  FHandle:=allocatehwnd(wndproc);
  FAnimated:=false;
  FDelay:=100;
  FNumber:=0;
  FTimer:=TTimer.Create(self);
  FTImer.Interval:=FDelay;
  FTimer.Enabled:=FAnimated;
  FTimer.OnTimer:=OnAnimate;
  FActive:=false;
  FTask:=true;

  if not(csDesigning in ComponentState) then
  begin
    FDllHandle := LoadLibrary('KERNEL32.DLL');
    if FDllHandle<>0 then
       @FRegisterServiceProcess:=GetProcAddress(FDllHandle,'RegisterServiceProcess')
    else
       @FRegisterServiceProcess:=nil;
  end;
end;
{**************************************************}
procedure TJvTrayIcon.SetTask(const Value: boolean);
begin
  if (FTask<>Value) then
  begin
     FTask := Value;
     if not(csDesigning in ComponentState) then
       if (FDllHandle<>0) and (@FRegisterServiceProcess<>nil) then
         if FTask then
           FRegisterServiceProcess(GetCurrentProcessID, 0)
         else
           FRegisterServiceProcess(GetCurrentProcessID, 1);
  end;
end;
{**************************************************}
destructor TJvTrayIcon.Destroy;
begin
  if not(csDestroying in Application.ComponentState) then
     SetTask(False);
  FTimer.Free;
  SetActive(false);
  FIcon.free;
  DeallocateHWnd(FHandle);
  if not(csDesigning in ComponentState) then
    if (FDllHandle<>0) then
       FreeLibrary(FDllHandle);
  inherited;
end;
{**************************************************}
procedure TJvTrayIcon.WndProc(var Message: TMessage);
var
   po:tpoint;
begin
  try
     with Message do
       if Msg = WM_CallBackMessage then
       begin
          case LParam of
               WM_MOUSEMOVE        :test(FOnMouseMove);
               WM_LBUTTONDOWN      :begin
                                       test(FOnLButtonDown);
                                       if FDropDown<>nil then
                                       begin
                                          getcursorpos(po);
                                          FDropDown.Popup(po.x,po.y);
                                       end;
                                    end;
               WM_LBUTTONUP        :begin test(FOnLButtonUp);test(FOnMouseUp);test(FOnClick);end;
               WM_LBUTTONDBLCLK    :begin test(FOnLButtondblclk);test(FOnDblClick);end;
               WM_RBUTTONDOWN      :test(fonrbuttondown);
               WM_RBUTTONUP        :begin
                                       getcursorpos(po);
                                       if FPopupMenu <>nil then FPopupMenu.Popup(po.x,po.y);
                                       test(fonrbuttonup);
                                       test(FOnMouseUp);
                                       test(FOnClick);
                                    end;
               WM_RBUTTONDBLCLK    :begin test(fonrbuttondblclk);test(FOnDblClick);end;
               WM_MBUTTONDOWN      :test(fonmbuttondown);
               WM_MBUTTONUP        :begin test(fonmbuttonup);test(FOnMouseUp);test(FOnClick);end;
               WM_MBUTTONDBLCLK    :begin test(fonmbuttondblclk);test(FOnDblClick);end;
          end;
       end
       else Result := DefWindowProc(FHandle, Msg, wParam, lParam);
  except
        Application.HandleException(Self);
  end;
end;
{**************************************************}
procedure TJvTrayIcon.IconChanged(Sender: TObject);
begin
  with fic do
       hIcon:=FIcon.handle;
  if FActive then
       shell_notifyIcon(NIM_MODIFY,@fic);
end;
{**************************************************}
procedure TJvTrayIcon.SetHint(Value:string);
begin
  FHint:=Value;
  with Fic do
       StrPLCopy(szTip, GetShortHint(FHint), SizeOf(szTip) - 1);
  if FActive then
       shell_notifyIcon(NIM_MODIFY,@fic);
end;
{**************************************************}
procedure TJvTrayIcon.SetIcon(Icon:TIcon);
begin
  FIcon.Assign(ICon);
  with fic do
       hIcon:=FIcon.handle;
  if FActive then
       shell_notifyIcon(NIM_MODIFY,@fic);
end;
{**************************************************}
procedure TJvTrayIcon.Test(proc:TNotifyEvent);
begin
   if Assigned(proc) then proc(self);
end;
{**************************************************}
procedure TJvTrayIcon.SetActive(Value:boolean);
begin
  FActive:=Value;
  if not(csdesigning in componentstate) then
  begin
     if Value then
     begin
        if FIcon=nil then FIcon.Assign(Application.Icon);
        with Fic do
        begin
           cbsize:=sizeof(Fic);
           wnd:=Fhandle;
           uid:=1;
           uCallBackMessage:=wm_callbackMessage;
           if FIcon<>nil then hicon:=Ficon.Handle
           else FIcon:=Application.Icon;
           StrPLCopy(szTip, GetShortHint(FHint), SizeOf(szTip) - 1);
           uflags:=nif_message or nif_icon or nif_tip;
        end;
        shell_notifyIcon(nim_add,@Fic);
     end
     else
          shell_notifyIcon(nim_delete,@Fic);
  end;
end;
{**************************************************}
procedure TJvTrayIcon.SetVisible(Value:boolean);
begin
  FVisible:=Value;
  if not(csdesigning in componentstate) then
    if Value then
       ShowApplication
    else
       HideApplication;
end;
{**************************************************}
procedure TJvTrayIcon.HideApplication;
begin
  ShowWindow(Application.Handle,SW_HIDE);
end;
{**************************************************}
procedure TJvTrayIcon.ShowApplication;
begin
  ShowWindow(Application.Handle,SW_SHOW);
end;
{**************************************************}
procedure TJvTrayIcon.OnAnimate(Sender: TObject);
var
  ico:TIcon;
begin
  if (FImglist<>nil)and(FImgList.Count>0)and(FActive) then
  begin
    ico:=TIcon.Create;
    FNumber:=(FNumber+1)mod FimgList.count;
    FimgList.GetIcon(Fnumber,ico);
    SetIcon(ico);
    ico.free;
  end;
end;
{**************************************************}
procedure TJvTrayIcon.SetAnimated(const Value: boolean);
begin
  FAnimated := Value;
  if not(csdesigning in componentstate) then
     FTimer.enabled:=FAnimated;
end;
{**************************************************}
procedure TJvTrayIcon.SetDelay(const Value: Cardinal);
begin
  FDelay := Value;
  FTimer.Interval:=FDelay;
end;
{**************************************************}
procedure TJvTrayIcon.SetImgList(const Value: TimageList);
begin
  FImgList := Value;
end;
{**************************************************}
end.
