{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvWaitingGradient.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvWaitingGradient;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, 
  StdCtrls;

type
  TJvWaitingThread = class(TThread)
  private
  protected
    procedure Draw;
    procedure Execute; override;
  public
    FDelay:cardinal;
    FOnDraw:TNotifyEvent;
  end;

  TJvWaitingGradient = class(TGraphicControl)
  private
    FTimerTag:Integer;
    FBitmap:TBitmap;
    FLeft,FWidth:Integer;
    FStarTColor,FEndColor:TColor;
    FStart,FEnd:TColor;
    FRect,FDRect:TRect;
    FLoading:boolean;
    FInterval: cardinal;
    FEnabled: boolean;
    FScroll:TJvWaitingThread;
    procedure Deplace(Sender: TObject);
    procedure CreeBitmap;
    procedure SetWidth(const Value: integer);
    procedure SetEndColor(const Value: TColor);
    procedure SetStarTColor(const Value: TColor);
    procedure SetInterval(const Value: cardinal);
    procedure SetEnable(const Value: boolean);
  protected
    procedure Paint;override;
    procedure Resize; override;
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property Color;
    property GradientWidth:Integer read FWidth write SetWidth;
    property StarTColor:TColor read FStart write SetStarTColor default clBtnFace;
    property EndColor:TColor read Fend write SetEndColor default clBlack;
    property Interval:cardinal read FInterval write SetInterval default 50;
    property Enabled:boolean read FEnabled write SetEnable default true;
    property Visible;
    property Align;
  end;

implementation

{**************************************************}
constructor TJvWaitingGradient.Create(AOwner: TComponent);
begin
   FLoading:=true;
   inherited;
   self.parent:=TWinControl(AOwner);
   self.Color:=clBtnFace;
   self.width:=100;
   self.height:=10;
   FInterval:=50;
   FEnabled := true;

   FStart:=clBtnFace;
   FEnd:=clBlack;
   FWidth:=50;
   FLeft:=-FWidth;
   FBitmap:=TBItmap.Create;
   CreeBitmap;
   FRect:=Rect(0,0,FWidth,self.height);
   FDRect:=Rect(0,0,FWidth,self.height);
   FTImertag:=0;

   FScroll:=TJvWaitingThread.Create(true);
   FScroll.FreeOnTerminate:=false;
   FScroll.FDelay:=FInterval;
   FScroll.FOnDraw:=Deplace;
   FScroll.Resume;

   FLoading:=false;
end;
{**************************************************}
procedure TJvWaitingGradient.CreeBitmap;
var
   i:Integer;
   j:real;
   Deltas:array [0..2] of real;  //R,G,B
   r:TRect;
   FSteps:Integer;
begin
   if FTimertag=0 then
   begin
      if (FStart<0) then FStarTColor:=GetSysColor(FStart and not( $80000000))
      else FStarTColor:=FStart;
      if (FEnd<0) then FEndColor:=GetSysColor(Fend and not( $80000000))
      else FEndColor:=FEnd;
   end
   else
   begin
      if (FStart<0) then FEndColor:=GetSysColor(FStart and not( $80000000))
      else FEndColor:=FStart;
      if (FEnd<0) then FStarTColor:=GetSysColor(Fend and not( $80000000))
      else FStarTColor:=FEnd;
   end;

   FBItmap.Width:=FWidth;
   FBitmap.Height:=self.Height;

   FSteps:=FWidth;
   if FSteps>self.width then FSteps:=self.Width;
   Deltas[0]:=(GetRValue(FEndColor)-GetRValue(FStarTColor))/FSteps;
   Deltas[1]:=(GetGValue(FEndColor)-GetGValue(FStarTColor))/FSteps;
   Deltas[2]:=(GetBValue(FEndColor)-GetBValue(FStarTColor))/FSteps;
   FBitmap.Canvas.brush.style:=bsSolid;
   j:=FWidth/FSteps;
   for i:=0 to FSteps do
   begin
      r.Top:=0;
      r.Bottom:=self.height;
      r.Left:=round(i*j);
      r.Right:=round((i+1)*j);
      FBitmap.Canvas.brush.Color:=RGB(Round(GetRValue(FStarTColor)+i*Deltas[0]),Round(GetGValue(FStarTColor)+i*Deltas[1]),Round(GetBValue(FStarTColor)+i*Deltas[2]));
      FBitmap.Canvas.FillRect(r);
   end;
end;
{**************************************************}
procedure TJvWaitingGradient.Deplace(Sender: TObject);
begin
   if FBitmap=nil then
      exit;
   try
      if FTimertag=0 then
      begin
         if FLeft+FWidth>=self.width then
         begin
            FTImertag:=1;
            CreeBitmap;
            FLeft:=Width;
         end
         else
            FLeft:=FLeft+2;
      end
      else
      begin
         if Fleft<=0 then
         begin
            FTImertag:=0;
            CreeBitmap;
            FLeft:=-FWidth;
         end
         else
            Fleft:=Fleft-2;
      end;
      FDRect.Left:=FLeft;
      FDRect.Right:=FLeft+FWidth;

      Paint;
   except
   end;
end;
{**************************************************}
destructor TJvWaitingGradient.Destroy;
begin
   FBitmap.Free;
   FBitmap:=nil;
   FScroll.Terminate;
   while not(FScroll.Terminated) do
      Application.ProcessMessages;
   FScroll.Free;
   inherited;
end;
{**************************************************}
procedure TJvWaitingGradient.Paint;
begin
   if Canvas<>nil then
   begin
      //paint
      Canvas.Brush.style := bsSolid;
      Canvas.Brush.Color := self.Color;
      Canvas.FillRect(Rect(0,0,FLeft,self.height));
      Canvas.FillRect(Rect(FLeft+FBitmap.Width,0,self.width,self.height));

      Canvas.CopyRect(FDRect,FBItmap.Canvas,FRect);
   end;
end;
{**************************************************}
procedure TJvWaitingGradient.Resize;
begin
   inherited;
   FRect:=Rect(0,0,FWidth,self.height);
   FDRect:=Rect(0,0,FWidth,self.height);
   if not(FLoading) then CreeBitmap;
end;
{**************************************************}
procedure TJvWaitingGradient.SetEnable(const Value: boolean);
begin
   FEnabled := Value;
   if Value then
      FScroll.Resume
   else
      FScroll.Suspend;
end;
{**************************************************}
procedure TJvWaitingGradient.SetEndColor(const Value: TColor);
begin
   Fend := Value;
   if not(FLoading) then CreeBitmap;
end;
{**************************************************}
procedure TJvWaitingGradient.SetInterval(const Value: cardinal);
begin
   FInterval := Value;
   FScroll.Fdelay:=Value;
end;
{**************************************************}
procedure TJvWaitingGradient.SetStarTColor(const Value: TColor);
begin
   FStart := Value;
   if not(FLoading) then CreeBitmap;
end;
{**************************************************}
procedure TJvWaitingGradient.SetWidth(const Value: integer);
begin
   if Value>0 then
   begin
      FWidth := Value;
      FLeft:=-FWidth;
      FRect:=Rect(0,0,FWidth,self.height);
      FDRect:=Rect(0,0,FWidth,self.height);
      if not(FLoading) then CreeBitmap;
   end
   else
     beep;
end;
{**************************************************}

///////////////////////////////////////////////////////////
// TJvWaitingThread
///////////////////////////////////////////////////////////

{**************************************************}
procedure TJvWaitingThread.Draw;
begin
   if Assigned(FOnDraw) then
      FOnDraw(nil)
end;
{**************************************************}
procedure TJvWaitingThread.Execute;
begin
   while not(Terminated) do
   begin
     SynChronize(Draw);
     sleep(FDelay);
   end;
end;
{**************************************************}
end.
